@@ -1,13 +1,14 @@
-__version__ = '0.7.2'
+__version__ = '0.7.3'
 
 from ._mpservice import ModelService, Modelet
+from ._mpservice import TimeoutError, EnqueueTimeout, TotalTimeout
 from ._mperror import MPError
 from ._server_process import ServerProcess
 
 
 __all__ = [
-    'MPError',
-    'Modelet',
-    'ModelService',
+    'Modelet', 'ModelService',
     'ServerProcess',
+    'MPError',
+    'TimeoutError', 'EnqueueTimeout', 'TotalTimeout',
 ]
